@@ -30,6 +30,8 @@
 
 #include "empathy-call-utils.h"
 
+#include <libempathy/empathy-request-util.h>
+
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
 
@@ -63,8 +65,7 @@ show_call_error (GError *error)
 
   dialog = gtk_message_dialog_new (NULL, 0,
       GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-      "%s",
-      get_error_display_message (error));
+      "%s", get_error_display_message (error));
 
   g_signal_connect_swapped (dialog, "response",
       G_CALLBACK (gtk_widget_destroy),
@@ -154,7 +155,8 @@ create_streamed_media_channel_cb (GObject *source,
 
   DEBUG ("Let's try with a Call channel");
   g_error_free (error);
-  tp_account_channel_request_create_channel_async (call_req, NULL, NULL,
+  tp_account_channel_request_create_channel_async (call_req,
+      EMPATHY_CALL_BUS_NAME, NULL,
       create_call_channel_cb,
       NULL);
 }
@@ -181,7 +183,8 @@ empathy_call_new_with_streams (const gchar *contact,
       streamed_media_request,
       timestamp);
 
-  tp_account_channel_request_create_channel_async (streamed_media_req, NULL, NULL,
+  tp_account_channel_request_create_channel_async (streamed_media_req,
+      EMPATHY_AV_BUS_NAME, NULL,
       create_streamed_media_channel_cb,
       call_req);
 
