@@ -62,12 +62,15 @@ void empathy_individual_linker_set_start_individual (
     EmpathyIndividualLinker *self,
     FolksIndividual *individual);
 
-GList * empathy_individual_linker_get_linked_personas (
+GeeSet * empathy_individual_linker_get_linked_personas (
     EmpathyIndividualLinker *self);
 
 gboolean empathy_individual_linker_get_has_changed (
     EmpathyIndividualLinker *self);
 
+void empathy_individual_linker_set_search_text (EmpathyIndividualLinker *self,
+    const gchar *search_text);
+
 G_END_DECLS
 
 #endif /* __EMPATHY_INDIVIDUAL_LINKER_H__ */
