@@ -61,11 +61,16 @@ typedef struct {
 	guint                 pages_loading;
 	/* Queue of GValue* containing an EmpathyMessage or string */
 	GQueue                message_queue;
+	/* Queue of owned gchar* of message token to remove unread
+	 * marker for when we lose focus. */
+	GQueue                acked_messages;
 	GtkWidget            *inspector_window;
 	GSettings            *gsettings_chat;
 	gboolean              has_focus;
 	gboolean              has_unread_message;
 	gboolean              allow_scrolling;
+	gchar                *variant;
+	gboolean              in_construction;
 } EmpathyThemeAdiumPriv;
 
 struct _EmpathyAdiumData {
@@ -78,6 +83,8 @@ struct _EmpathyAdiumData {
 	GHashTable *info;
 	guint version;
 	gboolean custom_template;
+	/* gchar* -> gchar* both owned */
+	GHashTable *date_format_cache;
 
 	/* HTML bits */
 	const gchar *template_html;
@@ -99,10 +106,12 @@ struct _EmpathyAdiumData {
 };
 
 static void theme_adium_iface_init (EmpathyChatViewIface *iface);
+static gchar * adium_info_dup_path_for_variant (GHashTable *info, const gchar *variant);
 
 enum {
 	PROP_0,
 	PROP_ADIUM_DATA,
+	PROP_VARIANT,
 };
 
 G_DEFINE_TYPE_WITH_CODE (EmpathyThemeAdium, empathy_theme_adium,
@@ -224,6 +233,27 @@ string_with_format (const gchar *format,
 	return g_string_free (result, FALSE);
 }
 
+static void
+theme_adium_load_template (EmpathyThemeAdium *theme)
+{
+	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
+	gchar                 *basedir_uri;
+	gchar                 *variant_path;
+	gchar                 *template;
+
+	priv->pages_loading++;
+	basedir_uri = g_strconcat ("file://", priv->data->basedir, NULL);
+	variant_path = adium_info_dup_path_for_variant (priv->data->info,
+		priv->variant);
+	template = string_with_format (priv->data->template_html,
+		variant_path, NULL);
+	webkit_web_view_load_html_string (WEBKIT_WEB_VIEW (theme),
+					  template, basedir_uri);
+	g_free (basedir_uri);
+	g_free (variant_path);
+	g_free (template);
+}
+
 static void
 theme_adium_match_newline (const gchar *text,
 			   gssize len,
@@ -404,6 +434,142 @@ static gchar *colors[] = {
 	"yellowgreen",
 };
 
+static const gchar *
+nsdate_to_strftime (EmpathyAdiumData *data, const gchar *nsdate)
+{
+	/* Convert from NSDateFormatter (http://www.stepcase.com/blog/2008/12/02/format-string-for-the-iphone-nsdateformatter/)
+	 * to strftime supported by g_date_time_format.
+	 * FIXME: table is incomplete, doc of g_date_time_format has a table of
+	 *        supported tags.
+	 * FIXME: g_date_time_format in GLib 2.28 does 0 padding by default, but
+	 *        in 2.29.x we have to explictely request padding with %0x */
+	static const gchar *convert_table[] = {
+		"a", "%p", // AM/PM
+		"A", NULL, // 0~86399999 (Millisecond of Day)
+
+		"cccc", "%A", // Sunday/Monday/Tuesday/Wednesday/Thursday/Friday/Saturday
+		"ccc", "%a", // Sun/Mon/Tue/Wed/Thu/Fri/Sat
+		"cc", "%u", // 1~7 (Day of Week)
+		"c", "%u", // 1~7 (Day of Week)
+
+		"dd", "%d", // 1~31 (0 padded Day of Month)
+		"d", "%d", // 1~31 (0 padded Day of Month)
+		"D", "%j", // 1~366 (0 padded Day of Year)
+
+		"e", "%u", // 1~7 (0 padded Day of Week)
+		"EEEE", "%A", // Sunday/Monday/Tuesday/Wednesday/Thursday/Friday/Saturday
+		"EEE", "%a", // Sun/Mon/Tue/Wed/Thu/Fri/Sat
+		"EE", "%a", // Sun/Mon/Tue/Wed/Thu/Fri/Sat
+		"E", "%a", // Sun/Mon/Tue/Wed/Thu/Fri/Sat
+
+		"F", NULL, // 1~5 (0 padded Week of Month, first day of week = Monday)
+
+		"g", NULL, // Julian Day Number (number of days since 4713 BC January 1)
+		"GGGG", NULL, // Before Christ/Anno Domini
+		"GGG", NULL, // BC/AD (Era Designator Abbreviated)
+		"GG", NULL, // BC/AD (Era Designator Abbreviated)
+		"G", NULL, // BC/AD (Era Designator Abbreviated)
+
+		"h", "%I", // 1~12 (0 padded Hour (12hr))
+		"H", "%H", // 0~23 (0 padded Hour (24hr))
+
+		"k", NULL, // 1~24 (0 padded Hour (24hr)
+		"K", NULL, // 0~11 (0 padded Hour (12hr))
+
+		"LLLL", "%B", // January/February/March/April/May/June/July/August/September/October/November/December
+		"LLL", "%b", // Jan/Feb/Mar/Apr/May/Jun/Jul/Aug/Sep/Oct/Nov/Dec
+		"LL", "%m", // 1~12 (0 padded Month)
+		"L", "%m", // 1~12 (0 padded Month)
+
+		"m", "%M", // 0~59 (0 padded Minute)
+		"MMMM", "%B", // January/February/March/April/May/June/July/August/September/October/November/December
+		"MMM", "%b", // Jan/Feb/Mar/Apr/May/Jun/Jul/Aug/Sep/Oct/Nov/Dec
+		"MM", "%m", // 1~12 (0 padded Month)
+		"M", "%m", // 1~12 (0 padded Month)
+
+		"qqqq", NULL, // 1st quarter/2nd quarter/3rd quarter/4th quarter
+		"qqq", NULL, // Q1/Q2/Q3/Q4
+		"qq", NULL, // 1~4 (0 padded Quarter)
+		"q", NULL, // 1~4 (0 padded Quarter)
+		"QQQQ", NULL, // 1st quarter/2nd quarter/3rd quarter/4th quarter
+		"QQQ", NULL, // Q1/Q2/Q3/Q4
+		"QQ", NULL, // 1~4 (0 padded Quarter)
+		"Q", NULL, // 1~4 (0 padded Quarter)
+
+		"s", "%S", // 0~59 (0 padded Second)
+		"S", NULL, // (rounded Sub-Second)
+
+		"u", "%Y", // (0 padded Year)
+
+		"vvvv", "%Z", // (General GMT Timezone Name)
+		"vvv", "%Z", // (General GMT Timezone Abbreviation)
+		"vv", "%Z", // (General GMT Timezone Abbreviation)
+		"v", "%Z", // (General GMT Timezone Abbreviation)
+
+		"w", "%W", // 1~53 (0 padded Week of Year, 1st day of week = Sunday, NB, 1st week of year starts from the last Sunday of last year)
+		"W", NULL, // 1~5 (0 padded Week of Month, 1st day of week = Sunday)
+
+		"yyyy", "%Y", // (Full Year)
+		"yyy", "%y", // (2 Digits Year)
+		"yy", "%y", // (2 Digits Year)
+		"y", "%Y", // (Full Year)
+		"YYYY", NULL, // (Full Year, starting from the Sunday of the 1st week of year)
+		"YYY", NULL, // (2 Digits Year, starting from the Sunday of the 1st week of year)
+		"YY", NULL, // (2 Digits Year, starting from the Sunday of the 1st week of year)
+		"Y", NULL, // (Full Year, starting from the Sunday of the 1st week of year)
+
+		"zzzz", NULL, // (Specific GMT Timezone Name)
+		"zzz", NULL, // (Specific GMT Timezone Abbreviation)
+		"zz", NULL, // (Specific GMT Timezone Abbreviation)
+		"z", NULL, // (Specific GMT Timezone Abbreviation)
+		"Z", "%z", // +0000 (RFC 822 Timezone)
+	};
+	const gchar *str;
+	GString *string;
+	guint i, j;
+
+	if (nsdate == NULL) {
+		return NULL;
+	}
+
+	str = g_hash_table_lookup (data->date_format_cache, nsdate);
+	if (str != NULL) {
+		return str;
+	}
+
+	/* Copy nsdate into string, replacing occurences of NSDateFormatter tags
+	 * by corresponding strftime tag. */
+	string = g_string_sized_new (strlen (nsdate));
+	for (i = 0; nsdate[i] != '\0'; i++) {
+		gboolean found = FALSE;
+
+		/* even indexes are NSDateFormatter tag, odd indexes are the
+		 * corresponding strftime tag */
+		for (j = 0; j < G_N_ELEMENTS (convert_table); j += 2) {
+			if (g_str_has_prefix (nsdate + i, convert_table[j])) {
+				found = TRUE;
+				break;
+			}
+		}
+		if (found) {
+			/* If we don't have a replacement, just ignore that tag */
+			if (convert_table[j + 1] != NULL) {
+				g_string_append (string, convert_table[j + 1]);
+			}
+			i += strlen (convert_table[j]) - 1;
+		} else {
+			g_string_append_c (string, nsdate[i]);
+		}
+	}
+
+	DEBUG ("Date format converted '%s' → '%s'", nsdate, string->str);
+
+	/* The cache takes ownership of string->str */
+	g_hash_table_insert (data->date_format_cache, g_strdup (nsdate), string->str);
+	return g_string_free (string, FALSE);
+}
+
+
 static void
 theme_adium_append_html (EmpathyThemeAdium *theme,
 			 const gchar       *func,
@@ -417,6 +583,7 @@ theme_adium_append_html (EmpathyThemeAdium *theme,
 		         gint64             timestamp,
 		         gboolean           is_backlog)
 {
+	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
 	GString     *string;
 	const gchar *cur = NULL;
 	gchar       *script;
@@ -474,16 +641,17 @@ theme_adium_append_html (EmpathyThemeAdium *theme,
 			replace = message;
 		} else if (theme_adium_match (&cur, "%time%") ||
 			   theme_adium_match_with_format (&cur, "%time{", &format)) {
-			/* FIXME: format is not exactly strftime.
-			 * See NSDateFormatter spec:
-			 * http://www.stepcase.com/blog/2008/12/02/format-string-for-the-iphone-nsdateformatter/
-			 */
+			const gchar *strftime_format;
+
+			strftime_format = nsdate_to_strftime (priv->data, format);
 			if (is_backlog) {
 				dup_replace = empathy_time_to_string_local (timestamp,
-					format ? format : EMPATHY_TIME_DATE_FORMAT_DISPLAY_SHORT);
+					strftime_format ? strftime_format :
+					EMPATHY_TIME_DATE_FORMAT_DISPLAY_SHORT);
 			} else {
 				dup_replace = empathy_time_to_string_local (timestamp,
-					format ? format : EMPATHY_TIME_FORMAT_DISPLAY_SHORT);
+					strftime_format ? strftime_format :
+					EMPATHY_TIME_FORMAT_DISPLAY_SHORT);
 			}
 			replace = dup_replace;
 		} else if (theme_adium_match (&cur, "%shortTime%")) {
@@ -569,26 +737,10 @@ theme_adium_append_event_escaped (EmpathyChatView *view,
 }
 
 static void
-theme_adium_remove_focus_marks (EmpathyThemeAdium *theme)
+theme_adium_remove_focus_marks (EmpathyThemeAdium *theme,
+    WebKitDOMNodeList *nodes)
 {
-	WebKitDOMDocument *dom;
-	WebKitDOMNodeList *nodes;
 	guint i;
-	GError *error = NULL;
-
-	dom = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (theme));
-	if (dom == NULL) {
-		return;
-	}
-
-	/* Get all nodes with focus class */
-	nodes = webkit_dom_document_query_selector_all (dom, ".focus", &error);
-	if (nodes == NULL) {
-		DEBUG ("Error getting focus nodes: %s",
-			error ? error->message : "No error");
-		g_clear_error (&error);
-		return;
-	}
 
 	/* Remove focus and firstFocus class */
 	for (i = 0; i < webkit_dom_node_list_get_length (nodes); i++) {
@@ -625,6 +777,36 @@ theme_adium_remove_focus_marks (EmpathyThemeAdium *theme)
 	}
 }
 
+static void
+theme_adium_remove_all_focus_marks (EmpathyThemeAdium *theme)
+{
+	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
+	WebKitDOMDocument *dom;
+	WebKitDOMNodeList *nodes;
+	GError *error = NULL;
+
+	if (!priv->has_unread_message)
+		return;
+
+	priv->has_unread_message = FALSE;
+
+	dom = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (theme));
+	if (dom == NULL) {
+		return;
+	}
+
+	/* Get all nodes with focus class */
+	nodes = webkit_dom_document_query_selector_all (dom, ".focus", &error);
+	if (nodes == NULL) {
+		DEBUG ("Error getting focus nodes: %s",
+			error ? error->message : "No error");
+		g_clear_error (&error);
+		return;
+	}
+
+	theme_adium_remove_focus_marks (theme, nodes);
+}
+
 static void
 theme_adium_append_message (EmpathyChatView *view,
 			    EmpathyMessage  *msg)
@@ -632,6 +814,7 @@ theme_adium_append_message (EmpathyChatView *view,
 	EmpathyThemeAdium     *theme = EMPATHY_THEME_ADIUM (view);
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
 	EmpathyContact        *sender;
+	TpMessage             *tp_msg;
 	TpAccount             *account;
 	gchar                 *body_escaped;
 	const gchar           *body;
@@ -721,10 +904,6 @@ theme_adium_append_message (EmpathyChatView *view,
 	message_classes = g_string_new ("message");
 	if (!priv->has_focus && !is_backlog) {
 		if (!priv->has_unread_message) {
-			/* This is the first message we receive since we lost
-			 * focus; remove previous unread marks. */
-			theme_adium_remove_focus_marks (theme);
-
 			g_string_append (message_classes, " firstFocus");
 			priv->has_unread_message = TRUE;
 		}
@@ -757,6 +936,20 @@ theme_adium_append_message (EmpathyChatView *view,
 	 * %status% - See %status% in theme_adium_append_html ()
 	 */
 
+	/* This is slightly a hack, but it's the only way to add
+	 * arbitrary data to messages in the HTML. We add another
+	 * class called "x-empathy-message-id-*" to the message. This
+	 * way, we can remove the unread marker for this specific
+	 * message later. */
+	tp_msg = empathy_message_get_tp_message (msg);
+	if (tp_msg != NULL) {
+		gchar *tmp = tp_escape_as_identifier (
+		    tp_message_get_token (tp_msg));
+		g_string_append_printf (message_classes,
+		    " x-empathy-message-id-%s", tmp);
+		g_free (tmp);
+	}
+
 	/* Define javascript function to use */
 	if (consecutive) {
 		func = priv->allow_scrolling ? "appendNextMessage" : "appendNextMessageNoScroll";
@@ -773,6 +966,9 @@ theme_adium_append_message (EmpathyChatView *view,
 			/* content */
 			html = consecutive ? priv->data->out_nextcontent_html : priv->data->out_content_html;
 		}
+
+		/* remove all the unread marks when we are sending a message */
+		theme_adium_remove_all_focus_marks (theme);
 	} else {
 		/* in */
 		if (is_backlog) {
@@ -847,14 +1043,8 @@ static void
 theme_adium_clear (EmpathyChatView *view)
 {
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (view);
-	gchar *basedir_uri;
 
-	priv->pages_loading++;
-	basedir_uri = g_strconcat ("file://", priv->data->basedir, NULL);
-	webkit_web_view_load_html_string (WEBKIT_WEB_VIEW (view),
-					  priv->data->template_html,
-					  basedir_uri);
-	g_free (basedir_uri);
+	theme_adium_load_template (EMPATHY_THEME_ADIUM (view));
 
 	/* Clear last contact to avoid trying to add a 'joined'
 	 * message when we don't have an insertion point. */
@@ -921,6 +1111,49 @@ theme_adium_copy_clipboard (EmpathyChatView *view)
 	webkit_web_view_copy_clipboard (WEBKIT_WEB_VIEW (view));
 }
 
+static void
+theme_adium_remove_mark_from_message (EmpathyThemeAdium *self,
+				      const gchar *token)
+{
+	WebKitDOMDocument *dom;
+	WebKitDOMNodeList *nodes;
+	gchar *class, *tmp;
+	GError *error = NULL;
+
+	dom = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (self));
+	if (dom == NULL) {
+		return;
+	}
+
+	tmp = tp_escape_as_identifier (token);
+	class = g_strdup_printf (".x-empathy-message-id-%s", tmp);
+	g_free (tmp);
+
+	/* Get all nodes with focus class */
+	nodes = webkit_dom_document_query_selector_all (dom, class, &error);
+	g_free (class);
+
+	if (nodes == NULL) {
+		DEBUG ("Error getting focus nodes: %s",
+			error ? error->message : "No error");
+		g_clear_error (&error);
+		return;
+	}
+
+	theme_adium_remove_focus_marks (self, nodes);
+}
+
+static void
+theme_adium_remove_acked_message_unread_mark_foreach (gpointer data,
+						      gpointer user_data)
+{
+	EmpathyThemeAdium *self = user_data;
+	gchar *token = data;
+
+	theme_adium_remove_mark_from_message (self, token);
+	g_free (token);
+}
+
 static void
 theme_adium_focus_toggled (EmpathyChatView *view,
 			   gboolean         has_focus)
@@ -928,11 +1161,46 @@ theme_adium_focus_toggled (EmpathyChatView *view,
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (view);
 
 	priv->has_focus = has_focus;
-	if (priv->has_focus) {
+	if (!priv->has_focus) {
+		/* We've lost focus, so let's make sure all the acked
+		 * messages have lost their unread marker. */
+		g_queue_foreach (&priv->acked_messages,
+				 theme_adium_remove_acked_message_unread_mark_foreach,
+				 view);
+		g_queue_clear (&priv->acked_messages);
+
 		priv->has_unread_message = FALSE;
 	}
 }
 
+static void
+theme_adium_message_acknowledged (EmpathyChatView *view,
+				  EmpathyMessage  *message)
+{
+	EmpathyThemeAdium *self = (EmpathyThemeAdium *) view;
+	EmpathyThemeAdiumPriv *priv = GET_PRIV (view);
+	TpMessage *tp_msg;
+
+	tp_msg = empathy_message_get_tp_message (message);
+
+	if (tp_msg == NULL) {
+		return;
+	}
+
+	/* We only want to actually remove the unread marker if the
+	 * view doesn't have focus. If we did it all the time we would
+	 * never see the unread markers, ever! So, we'll queue these
+	 * up, and when we lose focus, we'll remove the markers. */
+	if (priv->has_focus) {
+		g_queue_push_tail (&priv->acked_messages,
+				   g_strdup (tp_message_get_token (tp_msg)));
+		return;
+	}
+
+	theme_adium_remove_mark_from_message (self,
+					      tp_message_get_token (tp_msg));
+}
+
 static void
 theme_adium_context_menu_selection_done_cb (GtkMenuShell *menu, gpointer user_data)
 {
@@ -1053,6 +1321,7 @@ theme_adium_iface_init (EmpathyChatViewIface *iface)
 	iface->highlight = theme_adium_highlight;
 	iface->copy_clipboard = theme_adium_copy_clipboard;
 	iface->focus_toggled = theme_adium_focus_toggled;
+	iface->message_acknowledged = theme_adium_message_acknowledged;
 }
 
 static void
@@ -1118,6 +1387,11 @@ theme_adium_dispose (GObject *object)
 		priv->inspector_window = NULL;
 	}
 
+	if (priv->acked_messages.length > 0) {
+		g_queue_foreach (&priv->acked_messages, (GFunc) g_free, NULL);
+		g_queue_clear (&priv->acked_messages);
+	}
+
 	G_OBJECT_CLASS (empathy_theme_adium_parent_class)->dispose (object);
 }
 
@@ -1248,7 +1522,6 @@ static void
 theme_adium_constructed (GObject *object)
 {
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (object);
-	gchar                 *basedir_uri;
 	const gchar           *font_family = NULL;
 	gint                   font_size = 0;
 	WebKitWebView         *webkit_view = WEBKIT_WEB_VIEW (object);
@@ -1279,13 +1552,9 @@ theme_adium_constructed (GObject *object)
 			  object);
 
 	/* Load template */
-	priv->pages_loading = 1;
+	theme_adium_load_template (EMPATHY_THEME_ADIUM (object));
 
-	basedir_uri = g_strconcat ("file://", priv->data->basedir, NULL);
-	webkit_web_view_load_html_string (WEBKIT_WEB_VIEW (object),
-					  priv->data->template_html,
-					  basedir_uri);
-	g_free (basedir_uri);
+	priv->in_construction = FALSE;
 }
 
 static void
@@ -1300,6 +1569,9 @@ theme_adium_get_property (GObject    *object,
 	case PROP_ADIUM_DATA:
 		g_value_set_boxed (value, priv->data);
 		break;
+	case PROP_VARIANT:
+		g_value_set_string (value, priv->variant);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -1312,6 +1584,7 @@ theme_adium_set_property (GObject      *object,
 			  const GValue *value,
 			  GParamSpec   *pspec)
 {
+	EmpathyThemeAdium *theme = EMPATHY_THEME_ADIUM (object);
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (object);
 
 	switch (param_id) {
@@ -1319,6 +1592,9 @@ theme_adium_set_property (GObject      *object,
 		g_assert (priv->data == NULL);
 		priv->data = g_value_dup_boxed (value);
 		break;
+	case PROP_VARIANT:
+		empathy_theme_adium_set_variant (theme, g_value_get_string (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -1348,6 +1624,15 @@ empathy_theme_adium_class_init (EmpathyThemeAdiumClass *klass)
 							      G_PARAM_CONSTRUCT_ONLY |
 							      G_PARAM_READWRITE |
 							      G_PARAM_STATIC_STRINGS));
+	g_object_class_install_property (object_class,
+					 PROP_VARIANT,
+					 g_param_spec_string ("variant",
+							      "The theme variant",
+							      "Variant name for the theme",
+							      NULL,
+							      G_PARAM_CONSTRUCT |
+							      G_PARAM_READWRITE |
+							      G_PARAM_STATIC_STRINGS));
 
 	g_type_class_add_private (object_class, sizeof (EmpathyThemeAdiumPriv));
 }
@@ -1360,6 +1645,7 @@ empathy_theme_adium_init (EmpathyThemeAdium *theme)
 
 	theme->priv = priv;
 
+	priv->in_construction = TRUE;
 	g_queue_init (&priv->message_queue);
 	priv->allow_scrolling = TRUE;
 	priv->smiley_manager = empathy_smiley_manager_dup_singleton ();
@@ -1381,15 +1667,49 @@ empathy_theme_adium_init (EmpathyThemeAdium *theme)
 }
 
 EmpathyThemeAdium *
-empathy_theme_adium_new (EmpathyAdiumData *data)
+empathy_theme_adium_new (EmpathyAdiumData *data,
+			 const gchar *variant)
 {
 	g_return_val_if_fail (data != NULL, NULL);
 
 	return g_object_new (EMPATHY_TYPE_THEME_ADIUM,
 			     "adium-data", data,
+			     "variant", variant,
 			     NULL);
 }
 
+void
+empathy_theme_adium_set_variant (EmpathyThemeAdium *theme,
+				 const gchar *variant)
+{
+	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
+	gchar *variant_path;
+	gchar *script;
+
+	if (!tp_strdiff (priv->variant, variant)) {
+		return;
+	}
+
+	g_free (priv->variant);
+	priv->variant = g_strdup (variant);
+
+	if (priv->in_construction) {
+		return;
+	}
+
+	DEBUG ("Update view with variant: '%s'", variant);
+	variant_path = adium_info_dup_path_for_variant (priv->data->info,
+		priv->variant);
+	script = g_strdup_printf ("setStylesheet(\"mainStyle\",\"%s\");", variant_path);
+
+	webkit_web_view_execute_script (WEBKIT_WEB_VIEW (theme), script);
+
+	g_free (variant_path);
+	g_free (script);
+
+	g_object_notify (G_OBJECT (theme), "variant");
+}
+
 gboolean
 empathy_adium_path_is_valid (const gchar *path)
 {
@@ -1462,33 +1782,31 @@ adium_info_get_no_variant_name (GHashTable *info)
 	return name ? name : _("Normal");
 }
 
-static const gchar *
-adium_info_get_default_or_first_variant (GHashTable *info)
-{
-	const gchar *name;
-	GPtrArray *variants;
-
-	name = empathy_adium_info_get_default_variant (info);
-	if (name != NULL) {
-		return name;
-	}
-
-	variants = empathy_adium_info_get_available_variants (info);
-	g_assert (variants->len > 0);
-	return g_ptr_array_index (variants, 0);
-}
-
 static gchar *
 adium_info_dup_path_for_variant (GHashTable *info,
 				 const gchar *variant)
 {
 	guint version = adium_info_get_version (info);
 	const gchar *no_variant = adium_info_get_no_variant_name (info);
+	GPtrArray *variants;
+	guint i;
 
 	if (version <= 2 && !tp_strdiff (variant, no_variant)) {
 		return g_strdup ("main.css");
 	}
 
+	/* Verify the variant exists, fallback to the first one */
+	variants = empathy_adium_info_get_available_variants (info);
+	for (i = 0; i < variants->len; i++) {
+		if (!tp_strdiff (variant, g_ptr_array_index (variants, i))) {
+			break;
+		}
+	}
+	if (i == variants->len) {
+		DEBUG ("Variant %s does not exist", variant);
+		variant = g_ptr_array_index (variants, 0);
+	}
+
 	return g_strdup_printf ("Variants/%s.css", variant);
 
 }
@@ -1572,7 +1890,6 @@ empathy_adium_data_new_with_info (const gchar *path, GHashTable *info)
 	EmpathyAdiumData *data;
 	gchar            *template_html = NULL;
 	gchar            *footer_html = NULL;
-	gchar            *variant_path;
 	gchar            *tmp;
 
 	g_return_val_if_fail (empathy_adium_path_is_valid (path), NULL);
@@ -1585,6 +1902,8 @@ empathy_adium_data_new_with_info (const gchar *path, GHashTable *info)
 	data->info = g_hash_table_ref (info);
 	data->version = adium_info_get_version (info);
 	data->strings_to_free = g_ptr_array_new_with_free_func (g_free);
+	data->date_format_cache = g_hash_table_new_full (g_str_hash,
+		g_str_equal, g_free, g_free);
 
 	DEBUG ("Loading theme at %s", path);
 
@@ -1671,15 +1990,12 @@ empathy_adium_data_new_with_info (const gchar *path, GHashTable *info)
 		g_free (tmp);
 	}
 
-	variant_path = adium_info_dup_path_for_variant (info,
-		adium_info_get_default_or_first_variant (info));
-
 	/* Old custom templates had only 4 parameters.
 	 * New templates have 5 parameters */
 	if (data->version <= 2 && data->custom_template) {
 		tmp = string_with_format (template_html,
 			data->basedir,
-			variant_path,
+			"%@", /* Leave variant unset */
 			"", /* The header */
 			footer_html ? footer_html : "",
 			NULL);
@@ -1687,7 +2003,7 @@ empathy_adium_data_new_with_info (const gchar *path, GHashTable *info)
 		tmp = string_with_format (template_html,
 			data->basedir,
 			data->version <= 2 ? "" : "@import url( \"main.css\" );",
-			variant_path,
+			"%@", /* Leave variant unset */
 			"", /* The header */
 			footer_html ? footer_html : "",
 			NULL);
@@ -1697,7 +2013,6 @@ empathy_adium_data_new_with_info (const gchar *path, GHashTable *info)
 
 	g_free (template_html);
 	g_free (footer_html);
-	g_free (variant_path);
 
 	return data;
 }
@@ -1738,6 +2053,7 @@ empathy_adium_data_unref (EmpathyAdiumData *data)
 		g_free (data->default_outgoing_avatar_filename);
 		g_hash_table_unref (data->info);
 		g_ptr_array_unref (data->strings_to_free);
+		tp_clear_pointer (&data->date_format_cache, g_hash_table_unref);
 
 		g_slice_free (EmpathyAdiumData, data);
 	}
