@@ -121,12 +121,9 @@ gboolean    empathy_text_iter_backward_search           (const GtkTextIter*iter,
 							 GtkTextIter      *match_end,
 							 const GtkTextIter*limit);
 /* Windows */
-gboolean    empathy_window_get_is_visible               (GtkWindow        *window);
 void        empathy_window_present                      (GtkWindow *window);
 void        empathy_window_present_with_time            (GtkWindow        *window,
 							 guint32 timestamp);
-void        empathy_window_iconify                      (GtkWindow        *window,
-							 GtkStatusIcon    *status_icon);
 GtkWindow * empathy_get_toplevel_window                 (GtkWidget        *widget);
 
 /* URL */
@@ -152,6 +149,11 @@ GtkWidget * empathy_context_menu_new                    (GtkWidget *attach_to);
 
 gint64      empathy_get_current_action_time             (void);
 
+gboolean empathy_individual_match_string (
+    FolksIndividual *individual,
+    const gchar *text,
+    GPtrArray *words);
+
 G_END_DECLS
 
 #endif /*  __EMPATHY_UI_UTILS_H__ */
