@@ -238,65 +238,6 @@ empathy_xml_node_find_child_prop_value (xmlNodePtr   node,
 	return found;
 }
 
-GHashTable *
-empathy_call_create_streamed_media_request (EmpathyContact *contact,
-					    gboolean initial_audio,
-					    gboolean initial_video)
-{
-	return tp_asv_new (
-		TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
-			TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-		TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT,
-			TP_HANDLE_TYPE_CONTACT,
-		TP_PROP_CHANNEL_TARGET_HANDLE, G_TYPE_UINT,
-			empathy_contact_get_handle (contact),
-		TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_AUDIO, G_TYPE_BOOLEAN,
-			initial_audio,
-		TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO, G_TYPE_BOOLEAN,
-			initial_video,
-		NULL);
-}
-
-static void
-create_media_channel_cb (GObject *source,
-			 GAsyncResult *result,
-			 gpointer user_data)
-{
-	GError *error = NULL;
-
-	if (!tp_account_channel_request_create_channel_finish (TP_ACCOUNT_CHANNEL_REQUEST (source),
-							       result,
-							       &error)) {
-		DEBUG ("Failed to create StreamedMedia channel: %s", error->message);
-		g_error_free (error);
-	}
-}
-
-void
-empathy_call_new_with_streams (EmpathyContact *contact,
-			       gboolean initial_audio,
-			       gboolean initial_video,
-			       gint64 timestamp)
-{
-	GHashTable *request;
-	TpAccount *account;
-	TpAccountChannelRequest *req;
-
-	request = empathy_call_create_streamed_media_request (contact,
-							      initial_audio,
-							      initial_video);
-
-	account = empathy_contact_get_account (contact);
-
-	req = tp_account_channel_request_new (account, request, timestamp);
-
-	tp_account_channel_request_create_channel_async (req, EMPATHY_AV_BUS_NAME,
-							 NULL, create_media_channel_cb, NULL);
-
-	g_hash_table_unref (request);
-	g_object_unref (req);
-}
-
 const gchar *
 empathy_presence_get_default_message (TpConnectionPresenceType presence)
 {
@@ -776,18 +717,30 @@ empathy_folks_presence_type_to_tp (FolksPresenceType type)
 gboolean
 empathy_folks_individual_contains_contact (FolksIndividual *individual)
 {
-  GList *personas, *l;
+  GeeSet *personas;
+  GeeIterator *iter;
+  gboolean retval = FALSE;
 
   g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), FALSE);
 
   personas = folks_individual_get_personas (individual);
-  for (l = personas; l != NULL; l = l->next)
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (!retval && gee_iterator_next (iter))
     {
-      if (empathy_folks_persona_is_interesting (FOLKS_PERSONA (l->data)))
-        return (tpf_persona_get_contact (TPF_PERSONA (l->data)) != NULL);
-    }
+      FolksPersona *persona = gee_iterator_get (iter);
+      TpContact *contact = NULL;
 
-  return FALSE;
+      if (empathy_folks_persona_is_interesting (persona))
+        contact = tpf_persona_get_contact (TPF_PERSONA (persona));
+
+      g_clear_object (&persona);
+
+      if (contact != NULL)
+        retval = TRUE;
+    }
+  g_clear_object (&iter);
+
+  return retval;
 }
 
 /* TODO: this needs to be eliminated (and replaced in some cases with user
@@ -800,15 +753,17 @@ empathy_folks_individual_contains_contact (FolksIndividual *individual)
 EmpathyContact *
 empathy_contact_dup_from_folks_individual (FolksIndividual *individual)
 {
-  GList *personas, *l;
+  GeeSet *personas;
+  GeeIterator *iter;
   EmpathyContact *contact = NULL;
 
   g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
 
   personas = folks_individual_get_personas (individual);
-  for (l = personas; (l != NULL) && (contact == NULL); l = l->next)
+  iter = gee_iterable_iterator (GEE_ITERABLE (personas));
+  while (gee_iterator_next (iter) && (contact == NULL))
     {
-      TpfPersona *persona = l->data;
+      TpfPersona *persona = gee_iterator_get (iter);
 
       if (empathy_folks_persona_is_interesting (FOLKS_PERSONA (persona)))
         {
@@ -818,7 +773,9 @@ empathy_contact_dup_from_folks_individual (FolksIndividual *individual)
           contact = empathy_contact_dup_from_tp_contact (tp_contact);
           empathy_contact_set_persona (contact, FOLKS_PERSONA (persona));
         }
+      g_clear_object (&persona);
     }
+  g_clear_object (&iter);
 
   return contact;
 }
@@ -831,7 +788,7 @@ tp_channel_group_change_reason_from_folks_groups_change_reason (
 }
 
 TpfPersonaStore *
-empathy_get_persona_store_for_connection (TpConnection *connection)
+empathy_dup_persona_store_for_connection (TpConnection *connection)
 {
   FolksBackendStore *backend_store;
   FolksBackend *backend;
@@ -842,14 +799,14 @@ empathy_get_persona_store_for_connection (TpConnection *connection)
       "telepathy");
   if (backend != NULL)
     {
-      GHashTable *stores_hash;
-      GList *stores, *l;
+      GeeMap *stores_map;
+      GeeMapIterator *iter;
 
-      stores_hash = folks_backend_get_persona_stores (backend);
-      stores = g_hash_table_get_values (stores_hash);
-      for (l = stores; l != NULL && result == NULL; l = l->next)
+      stores_map = folks_backend_get_persona_stores (backend);
+      iter = gee_map_map_iterator (stores_map);
+      while (gee_map_iterator_next (iter))
         {
-          TpfPersonaStore *persona_store = TPF_PERSONA_STORE (l->data);
+          TpfPersonaStore *persona_store = gee_map_iterator_get_value (iter);
           TpAccount *account;
           TpConnection *conn_cur;
 
@@ -858,8 +815,7 @@ empathy_get_persona_store_for_connection (TpConnection *connection)
           if (conn_cur == connection)
             result = persona_store;
         }
-
-      g_list_free (stores);
+      g_clear_object (&iter);
     }
 
   g_object_unref (backend);
@@ -871,43 +827,58 @@ empathy_get_persona_store_for_connection (TpConnection *connection)
 gboolean
 empathy_connection_can_add_personas (TpConnection *connection)
 {
+  gboolean retval;
   FolksPersonaStore *persona_store;
 
   g_return_val_if_fail (TP_IS_CONNECTION (connection), FALSE);
 
   persona_store = FOLKS_PERSONA_STORE (
-      empathy_get_persona_store_for_connection (connection));
+      empathy_dup_persona_store_for_connection (connection));
 
-  return (folks_persona_store_get_can_add_personas (persona_store) ==
+  retval = (folks_persona_store_get_can_add_personas (persona_store) ==
       FOLKS_MAYBE_BOOL_TRUE);
+
+  g_clear_object (&persona_store);
+
+  return retval;
 }
 
 gboolean
 empathy_connection_can_alias_personas (TpConnection *connection)
 {
+  gboolean retval;
   FolksPersonaStore *persona_store;
 
   g_return_val_if_fail (TP_IS_CONNECTION (connection), FALSE);
 
   persona_store = FOLKS_PERSONA_STORE (
-      empathy_get_persona_store_for_connection (connection));
+      empathy_dup_persona_store_for_connection (connection));
 
-  return (folks_persona_store_get_can_alias_personas (persona_store) ==
+  retval = (folks_persona_store_get_can_alias_personas (persona_store) ==
       FOLKS_MAYBE_BOOL_TRUE);
+
+  g_clear_object (&persona_store);
+
+  return retval;
 }
 
 gboolean
 empathy_connection_can_group_personas (TpConnection *connection)
 {
+  gboolean retval;
   FolksPersonaStore *persona_store;
 
   g_return_val_if_fail (TP_IS_CONNECTION (connection), FALSE);
 
   persona_store = FOLKS_PERSONA_STORE (
-      empathy_get_persona_store_for_connection (connection));
+      empathy_dup_persona_store_for_connection (connection));
 
-  return (folks_persona_store_get_can_group_personas (persona_store) ==
+  retval = (folks_persona_store_get_can_group_personas (persona_store) ==
       FOLKS_MAYBE_BOOL_TRUE);
+
+  g_clear_object (&persona_store);
+
+  return retval;
 }
 
 gboolean
@@ -1031,4 +1002,3 @@ empathy_format_currency (gint         amount,
 
 	return money;
 }
-
